package com.icauda.article.pgm.tri.algo;

import static com.icauda.article.pgm.tri.util.Elements.permuter;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.icauda.article.pgm.tri.util.Elements;

public class FusionViaListeTri implements Tri {

	private static final Logger logger = LoggerFactory.getLogger(FusionViaListeTri.class);

	@Override
	public int[] trierCopy(int[] tab) {

		final int[] result = Arrays.copyOf(tab, tab.length);

		trier(result);

		return result;
	}

	@Override
	public void trier(int[] tab) {

		final List<Integer> liste = Elements.tabToList(tab);

		final List<Integer> result = trier(liste);

		Elements.ecraser(result, tab);

	}

	private List<Integer> trier(final List<Integer> liste) {

		if (liste.size() <= 1) {
			return liste;
		}

		// Separation en deux sous listes
		final int posCentre = liste.size() / 2;
		List<Integer> listeGauche = liste.subList(0, posCentre);
		List<Integer> listeDroite = liste.subList(posCentre, liste.size());

		// Tri des deux sous liste
		listeGauche = trier(listeGauche);
		listeDroite = trier(listeDroite);

		// Fusion
		final List<Integer> result = new ArrayList<>(liste.size());

		final Iterator<Integer> iterGauche = listeGauche.iterator();
		final Iterator<Integer> iterDroite = listeDroite.iterator();

		Integer g = next(iterGauche);
		Integer d = next(iterDroite);

		while (g != null || d != null) {

			if (d == null || g != null && g.compareTo(d) < 0) {
				result.add(g);
				g = next(iterGauche);
			} else {
				result.add(d);
				d = next(iterDroite);
			}
		}

		return result;
	}

	private static Integer next(final Iterator<Integer> iter) {
		return (iter.hasNext()) ? iter.next() : null;
	}
}
