package com.icauda.article.pgm.tri.algo;

import static com.icauda.article.pgm.tri.algo.Tabs.TAB;
import static com.icauda.article.pgm.tri.algo.Tabs.TAB_1000;
import static com.icauda.article.pgm.tri.algo.Tabs.TAB_QUASI_TRIE;
import static com.icauda.article.pgm.tri.algo.Tabs.TAB_TRIE;
import static com.icauda.article.pgm.tri.algo.Tabs.TAB_TRIE_ENVERS;

import java.util.Arrays;
import java.util.Random;

import org.junit.Assert;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runners.MethodSorters;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.icauda.article.pgm.tri.generator.Generator;
import com.icauda.article.pgm.tri.util.Elements;

@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public abstract class AbstractTriTest {

	private static final Logger logger = LoggerFactory.getLogger(AbstractTriTest.class);

	protected Tri tri;

	private boolean verifMode = true;

	/**
	 * Je ne veux que des valeurs entre 0 et 999 pour qu'il y ait plein de
	 * doublons et aussi que ce soit plus facile a lire.
	 */
	public static final int MAX = 999;

	protected static final String LINE = "*******************";

	private static final int TIMEOUT = 1 * 60 * 1000; // 5 min

	@Test
	public void testTriTabVide() {

		logger.debug("testTriTabVide");

		// Arrange
		final int[] tab = {};

		// Act - assert
		doTestTri(tab);
	}

	@Test
	public void testTriTabUnSeulElement() {

		logger.debug("testTriTabUnSeulElement");

		// Arrange
		final int[] tab = { 1 };

		// Act - assert
		doTestTri(tab);
	}

	@Test
	public void testTriTabDeuxElements() {

		logger.debug("testTriTabDeuxElements");

		// Arrange
		final int[] tab = { 2, 1 };
		Elements.printTab(tab);

		// Act - assert
		doTestTri(tab);
	}

	@Test
	public void testTriTabMelange() {

		logger.debug("testTriTabMelange");

		// Arrange
		final int[] tab = copyOf(TAB);

		// Act - assert
		doTestTri(tab);
	}

	@Test
	public void testTriTabQuasiTrie() {

		logger.debug("testTriTabQuasiTrie");

		// Arrange
		final int[] tab = copyOf(TAB_QUASI_TRIE);

		// Act - assert
		doTestTri(tab);
	}

	@Test
	public void testTriTabTrieEnvers() {

		logger.debug("testTriTabTrieEnvers");

		// Arrange
		final int[] tab = copyOf(TAB_TRIE_ENVERS);

		// Act - assert
		doTestTri(tab);
	}

	@Test
	public void testTriTabTrie() {

		logger.debug("testTriTabTrie");

		// Arrange
		final int[] tab = copyOf(TAB_TRIE);

		// Act - assert
		doTestTri(tab);
	}

	@Test(timeout = TIMEOUT)
	public void testTriTabAvec1_000Elements() {

		logger.debug("testTriTabAvec1_000Elements");

		// Arrange
		final int[] tab = copyOf(TAB_1000);

		// Act - assert
		doTestTri(tab);
	}

	@Test(timeout = TIMEOUT)
	public void testTriTabAleatoireAvec1_000Elements() {

		logger.debug("testTriTabAleatoireAvec1_000Elements");

		// Arrange
		final int nb = 1000; // 1 000

		// Act - assert
		doTestTriTabAleatoire(nb);
	}

	@Test(timeout = TIMEOUT)
	public void testTriTabAleatoireAvec10_000Elements() {

		logger.debug("testTriTabAleatoireAvec10_000Elements");

		// Arrange
		final int nb = 10 * 1000; // 10 000

		// Act - assert
		doTestTriTabAleatoire(nb);
	}

	@Test(timeout = TIMEOUT)
	public void testTriTabAleatoireAvec100_000Elements() {

		logger.debug("testTriTabAleatoireAvec100_000Elements");

		// Arrange
		final int nb = 100 * 1000; // 100 000

		// Act - assert
		doTestTriTabAleatoire(nb);
	}

	@Test(timeout = TIMEOUT)
	public void testTriTabAvec1000ElementsFromFile() {

		logger.debug("testTriTabAvec1000ElementsFromFile");

		// Arrange
		final String fileName = "src/test/resources/rand_1000.txt";

		final int[] tab = Generator.readFromFile(fileName);

		// Act - assert
		doTestTri(tab);
	}

	@Test(timeout = TIMEOUT)
	public void testTriTabAvec10_000ElementsFromFile() {

		logger.debug("testTriTabAvec10_000ElementsFromFile");

		// Arrange
		final String fileName = "src/test/resources/rand_10000.txt";

		final int[] tab = Generator.readFromFile(fileName);

		// Act - assert
		doTestTri(tab);
	}

	@Test(timeout = TIMEOUT)
	public void testTriTabAvec100_000ElementsFromFile() {

		logger.debug("testTriTabAvec100_000ElementsFromFile");

		// Arrange
		final String fileName = "src/test/resources/rand_100000.txt";

		final int[] tab = Generator.readFromFile(fileName);

		// Act - assert
		doTestTri(tab);
	}

	//@Test(timeout = TIMEOUT)
	public void testTriTabAvec1_000_000ElementsFromFile() {

		logger.debug("testTriTabAvec1_000_000ElementsFromFile");

		// Arrange
		final String fileName = "src/test/resources/rand_1000000.txt";

		final int[] tab = Generator.readFromFile(fileName);

		// Act - assert
		doTestTri(tab);
	}

	private void doTestTriTabAleatoire(final int nb) {

		logger.debug("doTestTriTabAleatoire");
		logger.debug("Taille : {} elements", nb);

		// Fabrication du tableau aleatoire
		final int[] tab = new int[nb];
		final Random rand = new Random();
		for (int i = 0; i < nb; i++) {
			tab[i] = rand.nextInt(MAX); // entre 0 et 999
		}

		// Act - assert
		doTestTri(tab);
	}

	protected void doTestTri(final int[] tab) {

		logger.debug("doTestTri");
		Elements.printTab(tab);
		
		int[] tab2 = null;
		if (verifMode) {
			tab2 = Arrays.copyOf(tab, tab.length);
		}

		// Act

		final long avant = System.currentTimeMillis();
		
		
		tri.trier(tab);
		Elements.printTab(tab);

		final long apres = System.currentTimeMillis();
		final long duree = apres - avant;

		logger.debug("Duree : {} ms", duree);

		// Assert
		assertMonotonie(tab);

		if (verifMode) {
			Arrays.sort(tab2);
			Assert.assertArrayEquals(tab2, tab);
		}
	}

	private static void assertMonotonie(final int[] tab) {
		int temp = Integer.MIN_VALUE;
		for (final int elt : tab) {
			Assert.assertTrue(temp <= elt);
			temp = elt;
		}
	}

	private int[] copyOf(final int[] tab) {
		return Arrays.copyOf(tab, tab.length);
	}

	public boolean isVerifMode() {
		return verifMode;
	}

	public void setVerifMode(boolean verifMode) {
		this.verifMode = verifMode;
	}

	// #############################################
	// D'autres tests a ne pas mettre dans l'article
	// #############################################

	// @Test
	// public void testTriCopyTabOriginalNonModifie() {
	//
	// logger.debug("testTriTabOriginalNonModifie");
	//
	// // Arrange
	// final int[] tab = TAB;
	//
	// // Act - assert
	// final int[] copy = Arrays.copyOf(tab, tab.length);
	// doTestTriCopy(tab);
	//
	// // Assert en plus
	// Assert.assertArrayEquals(copy, tab);
	// }
	//
	// @Test
	// public void testTriCopyTabVide() {
	//
	// logger.debug("testTriTabVide");
	//
	// // Arrange
	// final int[] tab = {};
	//
	// // Act - assert
	// doTestTriCopy(tab);
	// }
	//
	// @Test
	// public void testTriCopyTabUnSeulElement() {
	//
	// logger.debug("testTriTabUnSeulElement");
	//
	// // Arrange
	// final int[] tab = { 1 };
	//
	// // Act - assert
	// doTestTriCopy(tab);
	// }
	//
	// @Test
	// public void testTriCopyTabDeuxElements() {
	//
	// logger.debug("testTriTabDeuxElements");
	//
	// // Arrange
	// final int[] tab = { 2, 1 };
	//
	// // Act - assert
	// doTestTriCopy(tab);
	// }
	//
	// @Test
	// public void testTriCopyTabMelange() {
	//
	// logger.debug("testTriTabMelange");
	//
	// // Arrange
	// final int[] tab = TAB;
	//
	// // Act - assert
	// doTestTriCopy(tab);
	// }
	//
	// @Test
	// public void testTriCopyTabQuasiTrie() {
	//
	// logger.debug("testTriTabQuasiTrie");
	//
	// // Arrange
	// final int[] tab = TAB_QUASI_TRIE;
	//
	// // Act - assert
	// doTestTriCopy(tab);
	// }
	//
	// @Test
	// public void testTriCopyTabTrieEnvers() {
	//
	// logger.debug("testTriTabTrieEnvers");
	//
	// // Arrange
	// final int[] tab = TAB_TRIE_ENVERS;
	//
	// // Act - assert
	// doTestTriCopy(tab);
	// }
	//
	// @Test
	// public void testTriCopyTabTrie() {
	//
	// logger.debug("testTriTabTrie");
	//
	// // Arrange
	// final int[] tab = TAB_QUASI_TRIE;
	//
	// // Act - assert
	// doTestTriCopy(tab);
	// }
	//
	// @Test
	// public void testTriTabTrie1000Elements() {
	//
	// logger.debug("testTriTabTrie1000Elements");
	//
	// // Arrange
	// final int nb = 1000;
	//
	// // Act - assert
	// doTestTriCopyTabTrieNbElements(nb);
	// }
	//
	// @Test
	// public void testTriTabTrie10000Elements() {
	//
	// logger.debug("testTriTabTrie10000Elements");
	//
	// // Arrange
	// final int nb = 10000;
	//
	// // Act - assert
	// doTestTriCopyTabTrieNbElements(nb);
	// }
	//
	// @Test(timeout = TIMEOUT)
	// public void testTriTabTrie100_000Elements() {
	//
	// logger.debug("testTriTabTrie100_00Elements");
	//
	// // Arrange
	// final int nb = 100000; // 100 000
	//
	// // Act - assert
	// doTestTriCopyTabTrieNbElements(nb);
	// }
	//
	// @Test(timeout = TIMEOUT)
	// @Ignore
	// public void testTriTabTrie1_000_000Elements() {
	//
	// logger.debug("testTriTabTrie1_000_000Elements");
	//
	// // Arrange
	// final int nb = 1000000; // 1 000 000
	//
	// // Act - assert
	// doTestTriCopyTabTrieNbElements(nb);
	// }
	//
	// @Test(timeout = TIMEOUT)
	// public void testTriTabTrieEnvers100_000Elements() {
	//
	// logger.debug("testTriTabTrieEnvers100_000Elements");
	//
	// // Arrange
	// final int nb = 100000; // 100 000
	//
	// // Act - assert
	// doTestTriCopyTabTrieEnversNbElements(nb);
	// }
	//
	// @Test(timeout = TIMEOUT)
	// @Ignore
	// public void testTriTabTrieEnvers1_000_000Elements() {
	//
	// logger.debug("testTriTabTrieEnvers1_000_000Elements");
	//
	// // Arrange
	// final int nb = 1000000; // 1 000 000
	//
	// // Act - assert
	// doTestTriCopyTabTrieEnversNbElements(nb);
	// }
	//
	// private void doTestTriCopyTabTrieNbElements(final int nb) {
	// logger.debug("doTestTriTabTrieNbElements");
	//
	// // Arrange
	// final int[] tab = new int[nb];
	// for (int i = 0; i < nb; i++) {
	// tab[i] = i;
	// }
	//
	// // Act - assert
	// doTestTriCopy(tab);
	// }
	//
	// private void doTestTriCopyTabTrieEnversNbElements(final int nb) {
	// logger.debug("doTestTriTabTrieNbElements");
	//
	// // Arrange
	// int value = nb;
	// final int[] tab = new int[nb];
	// for (int i = 0; i < nb; i++) {
	// tab[i] = value--;
	// }
	//
	// // Act - assert
	// doTestTriCopy(tab);
	// }
	//
	// @Test
	// public void testTriTabAleatoireAvec100Elements() {
	//
	// logger.debug("testTriTabAleatoireAvec100Elements");
	//
	// // Arrange
	// final int nb = 100;
	//
	// // Act - assert
	// doTestTriCopyTabAleatoire(nb);
	// }
	//
	// @Test
	// public void testTriTabAleatoireAvec1000Elements() {
	//
	// logger.debug("testTriTabAleatoireAvec1000Elements");
	//
	// // Arrange
	// final int nb = 1000;
	//
	// // Act - assert
	// doTestTriCopyTabAleatoire(nb);
	// }
	//
	// @Test
	// public void testTriTabAleatoireAvec10000Elements() {
	//
	// logger.debug("testTriTabAleatoireAvec10000Elements");
	//
	// // Arrange
	// final int nb = 10000;
	//
	// // Act - assert
	// doTestTriCopyTabAleatoire(nb);
	// }
	//
	// @Test
	// public void testTriTabAleatoireAvec100000Elements() {
	//
	// logger.debug("testTriTabAleatoireAvec100000Elements");
	//
	// // Arrange
	// final int nb = 100000;
	//
	// // Act - assert
	// doTestTriCopyTabAleatoire(nb);
	// }
	//
	// private void doTestTriCopyTabAleatoire(final int nb) {
	//
	// logger.debug("doTestTriTabAleatoire");
	//
	// // Fabrication du tableau aleatoire
	// final int[] tab = new int[nb];
	// final Random rand = new Random();
	// for (int i = 0; i < nb; i++) {
	// tab[i] = rand.nextInt();
	// }
	//
	// // Act - assert
	// doTestTriCopy(tab);
	// }
	//
	// private void doTestTriCopy(final int[] tab) {
	//
	// logger.debug("doTestTriCopy");
	//
	// // Act
	// final long avant = System.currentTimeMillis();
	//
	// final int[] result = tri.trierCopy(tab);
	//
	// final long apres = System.currentTimeMillis();
	// final long duree = apres - avant;
	//
	// logger.debug("Duree : {} ms", duree);
	//
	// // Assert
	// Assert.assertEquals(tab.length, result.length);
	//
	// doTestMonotonie(result);
	// }

}
