package com.icauda.article.pgm.tri.generator;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class Generator {

	private static final Logger logger = LoggerFactory.getLogger(Generator.class);

	public static final String NEW_LINE = System.getProperty("line.separator");
	public static final int MAX = 999;

	public static int[] readFromFile(final String fileName) {
		logger.debug("readFromFile");
		logger.debug("fileName: {}", fileName);

		final File file = new File(fileName);

		if (!file.exists()) {
			logger.error("Le fichier n'existe pas");
		}

		final List<Integer> list = new LinkedList<>();

		try (final FileReader fr = new FileReader(file);//
				final BufferedReader br = new BufferedReader(fr);) {

			for (String line = br.readLine(); line != null; line = br.readLine()) {
				if (line.isEmpty()) {
					continue;
				}

				if (line.startsWith("#")) {
					continue;
				}

				list.add(Integer.valueOf(line));
			}

		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}

		final int[] tab = new int[list.size()];
		int i = 0;
		for (final int elt : list) {
			tab[i++] = elt;
		}

		return tab;

	}

	public static void generateFile(final String fileName, final int nb) {
		logger.debug("generateFile");
		logger.debug("fileName: {}", fileName);
		logger.debug("nb: {}", nb);

		final File file = new File(fileName);

		try (final FileWriter fw = new FileWriter(file); //
				final BufferedWriter bw = new BufferedWriter(fw);) {

			final Random random = new Random();

			for (int i = 0; i < nb; i++) {
				final Integer rand = random.nextInt(MAX);
				bw.write(rand.toString());
				bw.write(NEW_LINE);
			}

		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}

	}

	public static void main(String[] args) {
		logger.debug("Generator");

		final int[] tailles = { 10, 100, 1_000, 10_000, 100_000, 1_000_000 };

		logger.debug("Generation des fichiers.");
		for (final int nb : tailles) {
			final String fileName = "src/test/resources/rand_" + nb + ".txt";
			generateFile(fileName, nb);
		}

		final int[] tab100 = readFromFile("src/test/resources/rand_" + 100 + ".txt");

		logger.debug("Lecture des fichiers.");

		for (final int nb : tailles) {
			final String fileName = "src/test/resources/rand_" + nb + ".txt";
			final int[] tab = readFromFile(fileName);
			System.out.println(fileName);
			int i = 0;
			for (final int elt : tab) {
				System.out.print(elt);
				System.out.print("-");
				i++;
				if (i % 100 == 0) {
					System.out.println();
					i = 0;
				}
			}
		}

	}
}
